/*
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Bonneau and Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.debug.ui.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.model.IValue;
import org.eclipse.debug.core.model.IVariable;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.swt.graphics.Image;

import fr.obeo.acceleo.gen.debug.ui.AcceleoGenDebugUIMessages;
import fr.obeo.acceleo.gen.debug.ui.AcceleoGenDebugUiPlugin;
import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.gen.template.eval.ENodeList;

/**
 * Value of a AcceleoVariable.
 */
public class AcceleoValue extends AcceleoDebugElement implements IValue {

	private Object value;

	private int type;

	/**
	 * Constructor.
	 * 
	 * @param target
	 *            is the target
	 * @param value
	 *            is the value
	 * @param type
	 *            is the type of the value
	 */
	public AcceleoValue(AcceleoDebugTarget target, Object value, int type) {
		super(target);
		this.value = value;
		this.type = type;
	}

	/**
	 * Returns an image for the element, or null if a default image should be
	 * used.
	 * 
	 * @return an image or null
	 * @throws DebugException
	 */
	public Image getImage(Object element) throws DebugException {
		Image ret = null;
		if (type == AcceleoVariable.CLASS_TYPE) {
			ret = AcceleoGenDebugUiPlugin.getDefault().getImage("icons/class.gif"); //$NON-NLS-1$
		} else if (type == AcceleoVariable.ATTRIBUTE_TYPE) {
			ret = AcceleoGenDebugUiPlugin.getDefault().getImage("icons/attribute.gif"); //$NON-NLS-1$
		} else if (type == AcceleoVariable.REFERENCE_TYPE) {
			ret = AcceleoGenDebugUiPlugin.getDefault().getImage("icons/reference.gif"); //$NON-NLS-1$
		} else {
			if (value instanceof Boolean || (value instanceof ENode && ((ENode) value).isBoolean())) {
				ret = AcceleoGenDebugUiPlugin.getDefault().getImage("icons/boolean.gif"); //$NON-NLS-1$
			} else if (value instanceof Double || (value instanceof ENode && ((ENode) value).isDouble())) {
				ret = AcceleoGenDebugUiPlugin.getDefault().getImage("icons/double.gif"); //$NON-NLS-1$
			} else if (value instanceof EObject || (value instanceof ENode && ((ENode) value).isEObject())) {
				ret = AcceleoGenDebugUiPlugin.getDefault().getImage("icons/class.gif"); //$NON-NLS-1$
			} else if (value instanceof Integer || (value instanceof ENode && ((ENode) value).isInt())) {
				ret = AcceleoGenDebugUiPlugin.getDefault().getImage("icons/int.gif"); //$NON-NLS-1$
			} else if (value instanceof String || (value instanceof ENode && ((ENode) value).isString())) {
				ret = AcceleoGenDebugUiPlugin.getDefault().getImage("icons/string.gif"); //$NON-NLS-1$
			} else if (value instanceof List || value instanceof ENodeList || (value instanceof ENode && ((ENode) value).isList())) {
				ret = AcceleoGenDebugUiPlugin.getDefault().getImage("icons/list.gif"); //$NON-NLS-1$
			} else if (value == null || (value instanceof ENode && ((ENode) value).isNull())) {
				ret = AcceleoGenDebugUiPlugin.getDefault().getImage("icons/null.gif"); //$NON-NLS-1$
			} else {
				ret = null;
			}
		}
		return ret;
	}

	/* (non-Javadoc) */
	public String getReferenceTypeName() throws DebugException {
		String ret = ""; //$NON-NLS-1$
		if (value != null) {
			if (value instanceof ENode) {
				try {
					ENode node = (ENode) value;
					if (node.isBoolean()) {
						ret = "boolean"; //$NON-NLS-1$
					} else if (node.isDouble()) {
						ret = "double"; //$NON-NLS-1$
					} else if (node.isEObject()) {
						ret = node.getEObject().eClass().getName();
					} else if (node.isInt()) {
						ret = "int"; //$NON-NLS-1$
					} else if (node.isString()) {
						ret = "String"; //$NON-NLS-1$
					} else if (node.isList()) {
						ret = "List"; //$NON-NLS-1$
					}
				} catch (ENodeCastException nce) {
					IStatus status = new Status(IStatus.ERROR, "", IStatus.OK, nce.getMessage(), nce); //$NON-NLS-1$
					throw new DebugException(status);
				}
			} else if (value instanceof EObject) {
				ret = ((EObject) value).eClass().getName();
			} else {
				ret = value.getClass().getName().substring(value.getClass().getName().lastIndexOf('.') + 1);
			}
		}
		return ret;
	}

	/* (non-Javadoc) */
	public String getValueString() throws DebugException {
		String ret = ""; //$NON-NLS-1$
		try {
			if (value != null) {
				if (value instanceof EObject) {
					EObject eObject = (EObject) value;
					ret = getEObjectValue(eObject);
				} else if (value instanceof ENode) {
					ENode eNode = (ENode) value;
					if (eNode.isEObject()) {
						ret = getEObjectValue(eNode.getEObject());
					} else if (eNode.isList()) {
						ret = this.toSimpleString(eNode.getList().asList());
					} else {
						ret = eNode.getValue() != null ? eNode.getValue().toString() : ""; //$NON-NLS-1$
					}
				} else if (value instanceof ENodeList) {
					ret = this.toSimpleString(((ENodeList) value).asList());
				} else if (value instanceof Collection) {
					ret = this.toSimpleString((Collection) value);
				} else {
					ret = value.toString();
				}
			}
		} catch (ENodeCastException nce) {
			IStatus status = new Status(IStatus.ERROR, "", IStatus.OK, nce.getMessage(), nce); //$NON-NLS-1$
			throw new DebugException(status);
		}
		return ret;
	}

	/* (non-Javadoc) */
	public boolean isAllocated() throws DebugException {
		return true;
	}

	/* (non-Javadoc) */
	public IVariable[] getVariables() throws DebugException {
		IVariable[] ret = null;
		if (value != null) {
			if (value instanceof Collection) {
				ret = this.computeCollectionVariables((Collection) value);
			} else if (value instanceof ENode) {
				ret = this.computeENodeVariables((ENode) value);
			} else if (value instanceof ENodeList) {
				ret = this.computeCollectionVariables(((ENodeList) value).asList());
			} else if (value instanceof EObject) {
				ret = this.computeEObjectVariables((EObject) value);
			} else {
				ret = this.computeVariables(value);
			}
		} else {
			ret = new IVariable[0];
		}
		return ret;
	}

	/* (non-Javadoc) */
	public boolean hasVariables() throws DebugException {
		return getVariables().length > 0;
	}

	/**
	 * Computes variables of an EObject.
	 * 
	 * @param object
	 *            the EObject.
	 * @return the variables.
	 */
	private IVariable[] computeEObjectVariables(EObject object) {
		List variables = new LinkedList();
		List structuralFeatures = object.eClass().getEAllStructuralFeatures();
		for (Iterator iterFeatures = structuralFeatures.iterator(); iterFeatures.hasNext();) {
			EStructuralFeature currentFeature = (EStructuralFeature) iterFeatures.next();
			String featureName = currentFeature.getName();
			Object featureValue = object.eGet(currentFeature);
			if ((featureValue instanceof Collection && ((Collection) featureValue).size() > 0) || (!(featureValue instanceof Collection) && featureValue != null)) {
				variables.add(new AcceleoVariable((AcceleoDebugTarget) getDebugTarget(), featureName, featureValue, (currentFeature instanceof EReference) ? AcceleoVariable.REFERENCE_TYPE
						: AcceleoVariable.ATTRIBUTE_TYPE));
			}
		}
		return (IVariable[]) variables.toArray(new IVariable[variables.size()]);
	}

	/**
	 * Computes variables from a collection.
	 * 
	 * @param collection
	 *            the collection.
	 * @return the variables.
	 * @throws DebugException
	 */
	private IVariable[] computeCollectionVariables(Collection collection) throws DebugException {
		List variables = new LinkedList();
		try {
			for (Iterator iterObjects = collection.iterator(); iterObjects.hasNext();) {
				Object element = (Object) iterObjects.next();
				if (element instanceof ENode) {
					ENode eNode = (ENode) element;

					if (eNode.isEObject()) {
						variables.add(this.eObjectToVariable(eNode.getEObject()));
					} else if (eNode.isList()) {
						variables
								.add(new AcceleoVariable((AcceleoDebugTarget) getDebugTarget(), this.toSimpleString(eNode.getList().asList()), eNode.getList().asList(), AcceleoVariable.DEFAULT_TYPE));
					} else {
						variables.add(new AcceleoVariable((AcceleoDebugTarget) getDebugTarget(), "" + eNode.getValue(), eNode.getValue(), AcceleoVariable.DEFAULT_TYPE)); //$NON-NLS-1$
					}

				} else if (element instanceof EObject) {
					variables.add(this.eObjectToVariable((EObject) element));
				} else if (element instanceof Collection) {
					variables.add(new AcceleoVariable((AcceleoDebugTarget) getDebugTarget(), toSimpleString((Collection) element), element, AcceleoVariable.DEFAULT_TYPE));
				} else {
					variables.add(new AcceleoVariable((AcceleoDebugTarget) getDebugTarget(), "" + element, element, AcceleoVariable.DEFAULT_TYPE)); //$NON-NLS-1$
				}
			}
		} catch (ENodeCastException nce) {
			IStatus status = new Status(IStatus.ERROR, "", IStatus.OK, nce.getMessage(), nce); //$NON-NLS-1$
			throw new DebugException(status);
		}
		return (IVariable[]) variables.toArray(new IVariable[variables.size()]);
	}

	/**
	 * Computes variables from an ENode.
	 * 
	 * @param eNode
	 *            the ENode.
	 * @return the variables.
	 * @throws DebugException
	 */
	private IVariable[] computeENodeVariables(ENode eNode) throws DebugException {
		IVariable[] variables;
		try {
			if (eNode.isEObject()) {
				variables = this.computeEObjectVariables(eNode.getEObject());
			} else if (eNode.isList()) {
				variables = this.computeCollectionVariables(eNode.getList().asList());
			} else {
				variables = this.computeVariables(eNode.getValue());
			}
		} catch (ENodeCastException nce) {
			IStatus status = new Status(IStatus.ERROR, "", IStatus.OK, nce.getMessage(), nce); //$NON-NLS-1$
			throw new DebugException(status);
		}
		return variables;
	}

	/**
	 * Computes variables from an object.
	 * 
	 * @param object
	 *            the object
	 * @return an empty array.
	 */
	private IVariable[] computeVariables(Object object) {
		List variables = new LinkedList();
		return (IVariable[]) variables.toArray(new IVariable[variables.size()]);
	}

	/**
	 * Transforms an EObject into a variable.
	 * 
	 * @param eObject
	 *            the EOBject.
	 * @return the variable.
	 */
	private IVariable eObjectToVariable(EObject eObject) {
		String variableName = eObject.eClass().getName();
		return new AcceleoVariable((AcceleoDebugTarget) getDebugTarget(), variableName, eObject, AcceleoVariable.CLASS_TYPE);
	}

	/**
	 * Returns a value representing this EObject.
	 * 
	 * @param eObject
	 *            the EObject.
	 * @return a value represnting this EObject.
	 */
	private String getEObjectValue(EObject eObject) {
		String variableName = eObject.eClass().getName();
		EStructuralFeature feature = eObject.eClass().getEStructuralFeature("name"); //$NON-NLS-1$
		if (feature != null) {
			Object variableNameObject = eObject.eGet(feature);
			variableName = variableName + ':' + (variableNameObject != null ? variableNameObject.toString() : ""); //$NON-NLS-1$
		}
		return variableName;
	}

	/**
	 * Transforms a collection into a string.
	 * 
	 * @param collection
	 *            the collection.
	 * @return the string.
	 * @throws ENodeCastException
	 */
	private String toSimpleString(Collection collection) throws ENodeCastException {
		List strings = new ArrayList(collection.size());
		Iterator iterValues = collection.iterator();
		while (iterValues.hasNext()) {
			Object currentValue = iterValues.next();
			if (currentValue instanceof ENode) {
				ENode node = (ENode) currentValue;
				if (node.isEObject()) {
					EObject eObject = node.getEObject();
					String typeName = eObject.eClass().getName();
					String label = ""; //$NON-NLS-1$
					EStructuralFeature structuralFeature = eObject.eClass().getEStructuralFeature("name"); //$NON-NLS-1$
					if (structuralFeature != null) {
						Object nameObject = eObject.eGet(structuralFeature);
						label = (nameObject != null) ? nameObject.toString() : ""; //$NON-NLS-1$
					}
					strings.add(typeName + ':' + label);
				} else if (node.isList()) {
					strings.add(AcceleoGenDebugUIMessages.getString("AcceleoValue.CollectionName")); //$NON-NLS-1$
				} else {
					String typeName = node.getValue().getClass().getName().substring(node.getValue().getClass().getName().lastIndexOf('.') + 1);
					String label = node.getValue() != null ? node.getValue().toString() : ""; //$NON-NLS-1$
					strings.add(typeName + ':' + label);
				}
			} else if (currentValue instanceof EObject) {
				EObject eObject = (EObject) currentValue;
				String typeName = eObject.eClass().getName();
				String label = ""; //$NON-NLS-1$
				EStructuralFeature structuralFeature = eObject.eClass().getEStructuralFeature("name"); //$NON-NLS-1$
				if (structuralFeature != null) {
					Object nameObject = eObject.eGet(structuralFeature);
					label = (nameObject != null) ? nameObject.toString() : ""; //$NON-NLS-1$
				}
				strings.add(typeName + ':' + label);
			} else if (currentValue instanceof Collection) {
				strings.add(AcceleoGenDebugUIMessages.getString("AcceleoValue.CollectionName")); //$NON-NLS-1$
			} else if (currentValue instanceof ENodeList) {
				strings.add(AcceleoGenDebugUIMessages.getString("AcceleoValue.CollectionName")); //$NON-NLS-1$
			} else {
				String typeName = currentValue.getClass().getName().substring(currentValue.getClass().getName().lastIndexOf('.') + 1);
				String label = currentValue != null ? currentValue.toString() : ""; //$NON-NLS-1$
				strings.add(typeName + ':' + label);
			}
		}
		StringBuffer buffer = new StringBuffer("["); //$NON-NLS-1$
		Iterator iterStrings = strings.iterator();
		boolean first = true;
		while (iterStrings.hasNext()) {
			if (first) {
				first = false;
			} else {
				buffer.append(", "); //$NON-NLS-1$
			}
			buffer.append(iterStrings.next());
		}
		buffer.append(']');
		return buffer.toString();
	}
}
