/*
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Bonneau and Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.debug.ui.model;

import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.model.IValue;
import org.eclipse.debug.core.model.IVariable;

import fr.obeo.acceleo.gen.debug.ui.AcceleoGenDebugUIMessages;

/**
 * A variable in a Acceleo Stack Frame
 */
public class AcceleoVariable extends AcceleoDebugElement implements IVariable, Comparable {

	private String name;

	private Object value;

	private int type;

	/**
	 * The default type.
	 */
	protected static final int DEFAULT_TYPE = 0;

	/**
	 * The class type.
	 */
	protected static final int CLASS_TYPE = 1;

	/**
	 * The reference type.
	 */
	protected static final int REFERENCE_TYPE = 2;

	/**
	 * The attribute type.
	 */
	protected static final int ATTRIBUTE_TYPE = 3;

	/**
	 * Constructs a variable contained in the given stack frame with the given
	 * name.
	 * 
	 * @param frame
	 *            is the owning stack frame
	 * @param name
	 *            is the variable name
	 * @param aValue
	 *            is the value
	 * @param type
	 *            is the type of the variable
	 */
	public AcceleoVariable(AcceleoStackFrame frame, String name, Object aValue, int type) {
		this((AcceleoDebugTarget) (frame.getDebugTarget()), name, aValue, type);
	}

	/**
	 * Constructs a variable contained in the given debug target with the given
	 * name.
	 * 
	 * @param aDebugTarget
	 *            is the debug target
	 * @param name
	 *            is the variable name
	 * @param aValue
	 *            is the value
	 * @param type
	 *            is the type of the variable
	 */
	public AcceleoVariable(AcceleoDebugTarget aDebugTarget, String name, Object aValue, int type) {
		super(aDebugTarget);
		this.name = name;
		this.value = aValue;
		this.type = type;
	}

	/* (non-Javadoc) */
	public IValue getValue() throws DebugException {
		return new AcceleoValue((AcceleoDebugTarget) getDebugTarget(), value, type);
	}

	/* (non-Javadoc) */
	public String getName() throws DebugException {
		return name;
	}

	/* (non-Javadoc) */
	public String getReferenceTypeName() throws DebugException {
		String ret = AcceleoGenDebugUIMessages.getString("AcceleoVariable.UnknownTypeName"); //$NON-NLS-1$
		if (value != null) {
			ret = getSimpleName(value.getClass());
		}
		return ret;
	}

	private String getSimpleName(Class c) {
		String name = c.getName();
		int i = name.lastIndexOf("."); //$NON-NLS-1$
		if (i > -1) {
			name = name.substring(i + 1);
		}
		return name;
	}

	/* (non-Javadoc) */
	public boolean hasValueChanged() throws DebugException {
		return false;
	}

	/* (non-Javadoc) */
	public void setValue(String expression) throws DebugException {
	}

	/* (non-Javadoc) */
	public void setValue(IValue value) throws DebugException {
	}

	/* (non-Javadoc) */
	public boolean supportsValueModification() {
		return false;
	}

	/* (non-Javadoc) */
	public boolean verifyValue(String expression) throws DebugException {
		return false;
	}

	/* (non-Javadoc) */
	public boolean verifyValue(IValue value) throws DebugException {
		return false;
	}

	/* (non-Javadoc) */
	public int compareTo(Object arg0) {
		int ret = 0;
		if (arg0 instanceof AcceleoVariable) {
			AcceleoVariable variable = (AcceleoVariable) arg0;
			if (variable.name != null && name != null) {
				ret = name.compareTo(variable.name);
			}
		}
		return ret;
	}

}
