/*
 * Copyright (c) 2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.engine.context;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ILauncherManager;
import fr.obeo.acceleo.gen.template.eval.LaunchManager;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import org.acceleo.engine.template.core.impl.spec.TService;
import org.acceleo.engine.template.core.impl.spec.TTemplate;
import org.acceleo.engine.template.impl.spec.IEvaluable;
import org.acceleo.parser.template.TemplateSyntaxException;
import org.acceleo.parser.template.expressions.ExpressionParser;
import org.acceleo.template.Resource;
import org.acceleo.template.core.Script;
import org.acceleo.template.core.Template;
import org.acceleo.tools.strings.Int2;
import org.eclipse.emf.ecore.EObject;

/**
 * Eval context. You must create one per thread.
 * 
 * @author David Bonneau
 * 
 */
public final class EvalContext {

	/** Stack for the template contexts. */
	private final Stack<TTemplate> templateContexts = new Stack<TTemplate>();

	/** context stacks; */
	private final Map<Object, Stack<Object>> context = new HashMap<Object, Stack<Object>>();

	/** List of visited EObjects for a given IEvaluable during the generation. */
	protected Map<IEvaluable, List<EObject>> visited = new HashMap<IEvaluable, List<EObject>>();

	/** Launch manager */
	ILauncherManager launchManager;

	/**
	 * Start a new template context for the given Template.
	 * 
	 * @param script
	 *            the Script to start
	 */
	public void startTemplate(Template template) {
		// TODO g�rer les erreurs si on est encore sur le m�me template
		// TODO get properties container from template
		templateContexts.push((TTemplate) template);
	}

	/**
	 * End the last started Script.
	 */
	public void endTemplate() {
		// TODO Manage error if there is an empty context
		templateContexts.pop();
	}

	/**
	 * Getter for the current template context.
	 * 
	 * @return the current template context.
	 */
	public TTemplate getCurrentTemplateContext() {
		return templateContexts.peek();
	}

	/**
	 * Get the Script with a given name that can be applied to the given
	 * EClassifier.
	 * 
	 * @param eClass
	 *            the EClassifier on witch the Script can be applied
	 * @param name
	 *            the name of the script we are looking for
	 * @return the matching Script if founded, null otherwise
	 */
	public Script getScript(EObject eObj, String name) {
		Script script = null;
		// search in specifics templates
		for (int i = 0; i < templateContexts.size() - 1 && script == null; ++i) {
			script = templateContexts.get(i).getScript(eObj, name);
		}

		// search in current template
		if (script == null) {
			script = getCurrentTemplateContext().getScript(eObj, name);
		}

		// search in imported templates
		if (script == null) {
			final Iterator<Resource> it = getCurrentTemplateContext().getImports().iterator();
			while (script == null && it.hasNext()) {
				final Resource imp = it.next();
				if (imp instanceof Template) {
					script = ((TTemplate) imp).getScript(eObj, name);
				}
			}
		}

		return script;
	}

	/**
	 * Getter for the currently usable services.
	 * 
	 * @return the list of currently usable services
	 */
	public List<TService> getCurrentServices() {
		final List<TService> ret = new ArrayList<TService>();
		ret.addAll(getCurrentTemplateContext().getServices());
		return ret;
	}

	/**
	 * Compilation of a string representing an Acceleo expression.
	 * 
	 * @param offset
	 *            current ENode
	 * @param expression
	 *            the string representing the expression
	 * @return result of the compilation
	 * @throws TemplateSyntaxException
	 *             If the expression is not valid.
	 */
	public IEvaluable compile(int offset, String expression) throws TemplateSyntaxException {
		return (IEvaluable) ExpressionParser.fromString(offset, expression, new Int2(0, expression.length()), null);
	}

	public ILauncherManager getLaunchManager() {
		if (launchManager == null) {
			launchManager = LaunchManager.create("run", false); //$NON-NLS-1$
		}
		return launchManager;
	}

	public void setLaunchManager(ILauncherManager launchManager) {
		this.launchManager = launchManager;
	}

	/**
	 * Returns the value to which the context of this generator maps the
	 * specified key.
	 * 
	 * @param key
	 *            whose associated value is to be returned
	 * @return the value to which the context of this generator maps the
	 *         specified key
	 */
	public Object contextPeek(Object key) {
		Object res = null;
		final Stack<Object> stack = context.get(key);
		if (stack != null && !stack.isEmpty()) {
			final Object result = stack.peek();
			if (result instanceof ENode) {
				res = ((ENode) result).copy();
			} else if (result instanceof ENode[]) {
				final ENode[] copy = new ENode[((ENode[]) result).length];
				for (int i = 0; i < copy.length; i++) {
					copy[i] = ((ENode[]) result)[i].copy();
				}
				res = copy;
			} else {
				res = result;
			}
		}
		return res;
	}

	/**
	 * Returns the value to which the context of this generator maps the
	 * specified key at the given index.
	 * 
	 * @param key
	 *            is the key
	 * @param index
	 *            is the index
	 * @return the value to which the context of this generator maps the
	 *         specified key
	 */
	public Object contextAt(Object key, int index) {
		Object res = null;
		final Stack<Object> stack = context.get(key);
		if (stack != null) {
			if (index >= 0 && index < stack.size()) {
				final Object result = stack.elementAt(stack.size() - index - 1);
				if (result instanceof ENode) {
					res = ((ENode) result).copy();
				} else if (result instanceof ENode[]) {
					final ENode[] copy = new ENode[((ENode[]) result).length];
					for (int i = 0; i < copy.length; i++) {
						copy[i] = ((ENode[]) result)[i].copy();
					}
					res = copy;
				}
			}
		}
		return res;
	}

	/**
	 * Associates the specified value with the specified key in the context of
	 * this generator.
	 * 
	 * @param key
	 *            with which the specified value is to be associated
	 * @param value
	 *            to be associated with the specified key
	 */
	public void contextPush(Object key, Object value) {
		Stack<Object> stack = context.get(key);
		if (stack == null) {
			stack = new Stack<Object>();
			context.put(key, stack);
		}
		stack.push(value);
	}

	/**
	 * Deletes the last specified value for the specified key in the context of
	 * this generator.
	 * 
	 * @param key
	 *            is the key
	 */
	public void contextPop(Object key) {
		final Stack<Object> stack = context.get(key);
		if (stack != null) {
			stack.pop();
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isRecursive(IEvaluable evaluable, EObject eObj) {
		final List<EObject> list = visited.get(evaluable);
		return list != null && list.contains(eObj);
	}

	/**
	 * {@inheritDoc}
	 */
	public List<EObject> getVisitedEObjects(IEvaluable evaluable) {
		List<EObject> res = visited.get(evaluable);
		if (res == null) {
			res = new ArrayList<EObject>();
			visited.put(evaluable, res);
		}
		return res;
	}
}
